use std::fmt::Display;

use crate::std::atomic::*;

#[derive(Clone)]
pub struct Plain {
    data: Vec<u8>,
}

#[derive(Debug)]
pub enum PlainParseError {}

impl Display for PlainParseError {
    fn fmt(&self, _f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match *self {}
    }
}

impl Error for PlainParseError {}

impl Serializable for Plain {
    fn serialize(&self, serializer: &mut dyn Serializer) {
        serializer.write(&self.data)
    }
}

impl Atomic for Plain {
    type ParseError = PlainParseError;

    fn deserialize(deserializer: &mut dyn Deserializer) -> Result<Self, Self::ParseError> {
        Ok(Plain::from_slice(deserializer.read_all()))
    }

    fn unexpected_tail(tail: &[u8]) -> Self::ParseError {
        panic!(
            "Plain must use read_all, therefore there must not be any extra tail (received {} bytes).",
            tail.len()
        )
    }
}

impl Plain {
    pub fn from_slice(slice: &[u8]) -> Self {
        Plain { data: slice.into() }
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::std::*;

    #[test]
    fn test_plain() -> Result<(), PlainParseError> {
        let plain = Plain::parse_slice(b"slice")?;
        assert_eq!(plain.bytes().as_slice(), b"slice");
        Ok(())
    }
}
